/*
 * utils/mountd/rmtab.c
 *
 * Manage the rmtab file for mountd.
 *
 * Copyright (C) 1995, 1996 Olaf Kirch <okir@monad.swb.de>
 */

#include "config.h"

#include <sys/stat.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <string.h>
#include "xmalloc.h"
#include "misc.h"
#include "exportfs.h"
#include "xlog.h"
#include "mountd.h"

static mountlist	mlist = NULL;

void
mountlist_add(nfs_export *exp, const char *path)
{
  mountlist m;

  for (m = mlist; m; m = m->ml_next)
    if (strcmp (m->ml_hostname, exp->m_client->m_hostname) == 0
	&& strcmp(m->ml_directory, path) == 0)
      return;

  m = (mountlist) xmalloc(sizeof(*m));
  if (m)
    {
      m->ml_hostname = xstrdup(exp->m_client->m_hostname);
      m->ml_directory = xstrdup(path);
      m->ml_next = mlist;
      mlist = m;
    }

  if (!m || !m->ml_hostname || !m->ml_directory)
    xlog (L_ERROR, "out of memory in mountlist_add()");
}

void
mountlist_del(nfs_export *exp, const char *path)
{
  mountlist m, *mp;

  if (!mlist)
    {
      xlog (L_ERROR, "empty mountlist in mountlist_del()");
      return;
    }

#if 0
  if (strcmp (mlist->ml_hostname, exp->m_client->m_hostname) == 0
      && strcmp(mlist->ml_directory, path) == 0)
    {
      /* Find a match. */
      m = mlist->ml_next;
      xfree (mlist->ml_hostname);
      xfree (mlist->ml_directory);
      xfree (mlist);
      mlist = m;
      return;
    }

  for (m = mlist, n = m->ml_next; n; m = m->ml_next, n = m->ml_next)
    if (strcmp (n->ml_hostname, exp->m_client->m_hostname) == 0
	&& strcmp(n->ml_directory, path) == 0)
      {
	/* Find a match. */
	m->ml_next = n->ml_next;
	xfree (n->ml_hostname);
	xfree (n->ml_directory);
	xfree (n);
	return;
      }
#else
   mp = &mlist;
   while ((m = *mp) != NULL) 
    if (strcmp (m->ml_hostname, exp->m_client->m_hostname) == 0
	&& strcmp(m->ml_directory, path) == 0)
      {
	/* Find a match. */
	*mp = m->ml_next; 
	xfree (m->ml_hostname);
	xfree (m->ml_directory);
	xfree (m);
	return;
      }
    else
      mp = &(m->ml_next);
#endif

  xlog (L_ERROR, "cannot find %s:%s in mountlist_del()",
  	exp->m_client->m_hostname, path);
}

void
mountlist_del_all(struct sockaddr_in *sin)
{
  struct in_addr addr = sin->sin_addr;
  struct hostent *hp;
  nfs_export *exp;
  mountlist m, n;

  if (!mlist)
    {
      xlog (L_ERROR, "empty mountlist in mountlist_del_all()");
      return;
    }

  if (!(hp = gethostbyaddr((char *)&addr, sizeof(addr), AF_INET)))
    {
      xlog(L_ERROR, "can't get hostname of %s", inet_ntoa(addr));
      return;
    }
  else
    hp = hostent_dup (hp);

  while (strcmp (mlist->ml_hostname, hp->h_name) == 0)
    {
      /* Find a match. */
      exp = auth_authenticate("umountall", sin, mlist->ml_directory, 1);
      if (exp)
	export_reset(exp);
      else
	xlog(L_NOTICE, "%s:%s is no longer exported.",
	     mlist->ml_hostname, mlist->ml_directory);

      m = mlist->ml_next;
      xfree (mlist->ml_hostname);
      xfree (mlist->ml_directory);
      xfree (mlist);
      mlist = m;
    }

  for (m = mlist, n = m->ml_next; n; m = m->ml_next, n = m->ml_next)
    if (strcmp (n->ml_hostname, hp->h_name) == 0)
      {
	/* Find a match. */
	exp = auth_authenticate("umountall", sin, n->ml_directory, 1);
	if (exp)
	  export_reset(exp);
	else
	  xlog(L_NOTICE, "%s:%s is no longer exported.",
	       n->ml_hostname, n->ml_directory);

	m->ml_next = n->ml_next;
	xfree (n->ml_hostname);
	xfree (n->ml_directory);
	xfree (n);
      }

  xfree (hp);
}

mountlist
mountlist_list(void)
{
  return mlist;
}
